@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: BSEncodeByteStream.cpp,v 1.9 2007/03/25 20:48:29 leonb Exp $
-// $Name: release_3_5_22 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -74,6 +71,7 @@
 #include "GOS.h"
 #endif
 
+#include <stddef.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -846,10 +844,12 @@ BSByteStream::Encode::encode()
       BitContext *cx = ctx;
       b = (mtfno==0);
       zp.encoder(b, cx[ctxid]);
-      if (b) goto rotate; cx+=CTXIDS;
+      if (b) goto rotate;
+      cx+=CTXIDS;
       b = (mtfno==1);
       zp.encoder(b, cx[ctxid]);
-      if (b) goto rotate; cx+=CTXIDS;
+      if (b) goto rotate;
+      cx+=CTXIDS;
       b = (mtfno<4);
       zp.encoder(b, cx[0]);
       if (b) { encode_binary(zp,cx+1,1,mtfno-2); goto rotate; } 
