@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: GBitmap.cpp,v 1.14 2007/07/14 01:21:56 leonb Exp $
-// $Name: release_3_5_22 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -69,9 +66,10 @@
 #include "GString.h"
 #include "GThreads.h"
 #include "GException.h"
+#include <stddef.h>
+#include <stdlib.h>
 #include <string.h>
 
-// File "$Id: GBitmap.cpp,v 1.14 2007/07/14 01:21:56 leonb Exp $"
 // - Author: Leon Bottou, 05/1997
 
 
@@ -100,16 +98,18 @@ GBitmap::destroy(void)
 
 GBitmap::GBitmap()
   : nrows(0), ncolumns(0), border(0), 
-    bytes_per_row(0), grays(0), bytes(0), gbytes_data(bytes_data), 
-    grle(rle), grlerows(rlerows), rlelength(0),
+    bytes_per_row(0), grays(0), bytes(0),
+    bytes_data(0), gbytes_data(bytes_data), 
+    rle(0), grle(rle), rlerows(0), grlerows(rlerows), rlelength(0),
     monitorptr(0)
 {
 }
 
 GBitmap::GBitmap(int nrows, int ncolumns, int border)
   : nrows(0), ncolumns(0), border(0), 
-    bytes_per_row(0), grays(0), bytes(0), gbytes_data(bytes_data), 
-    grle(rle), grlerows(rlerows), rlelength(0),
+    bytes_per_row(0), grays(0), bytes(0),
+    bytes_data(0), gbytes_data(bytes_data), 
+    rle(0), grle(rle), rlerows(0), grlerows(rlerows), rlelength(0),
     monitorptr(0)
 {
   G_TRY
@@ -126,8 +126,9 @@ GBitmap::GBitmap(int nrows, int ncolumns, int border)
 
 GBitmap::GBitmap(ByteStream &ref, int border)
   : nrows(0), ncolumns(0), border(0), 
-    bytes_per_row(0), grays(0), bytes(0), gbytes_data(bytes_data),
-    grle(rle), grlerows(rlerows), rlelength(0),
+    bytes_per_row(0), grays(0), bytes(0),
+    bytes_data(0), gbytes_data(bytes_data),
+    rle(0), grle(rle), rlerows(0), grlerows(rlerows), rlelength(0),
     monitorptr(0)
 {
   G_TRY
@@ -144,8 +145,9 @@ GBitmap::GBitmap(ByteStream &ref, int border)
 
 GBitmap::GBitmap(const GBitmap &ref)
   : nrows(0), ncolumns(0), border(0), 
-    bytes_per_row(0), grays(0), bytes(0), gbytes_data(bytes_data), 
-    grle(rle), grlerows(rlerows), rlelength(0),
+    bytes_per_row(0), grays(0), bytes(0),
+    bytes_data(0), gbytes_data(bytes_data), 
+    rle(0), grle(rle), rlerows(0), grlerows(rlerows), rlelength(0),
     monitorptr(0)
 {
   G_TRY
@@ -162,8 +164,9 @@ GBitmap::GBitmap(const GBitmap &ref)
 
 GBitmap::GBitmap(const GBitmap &ref, int border)
   : nrows(0), ncolumns(0), border(0), 
-    bytes_per_row(0), grays(0), bytes(0), gbytes_data(bytes_data),
-    grle(rle), grlerows(rlerows), rlelength(0),
+    bytes_per_row(0), grays(0), bytes(0),
+    bytes_data(0), gbytes_data(bytes_data),
+    rle(0), grle(rle), rlerows(0), grlerows(rlerows), rlelength(0),
     monitorptr(0)
 {
   G_TRY
@@ -181,8 +184,9 @@ GBitmap::GBitmap(const GBitmap &ref, int border)
 
 GBitmap::GBitmap(const GBitmap &ref, const GRect &rect, int border)
   : nrows(0), ncolumns(0), border(0), 
-    bytes_per_row(0), grays(0), bytes(0), gbytes_data(bytes_data),
-    grle(rle), grlerows(rlerows), rlelength(0),
+    bytes_per_row(0), grays(0), bytes(0),
+    bytes_data(0), gbytes_data(bytes_data),
+    rle(0), grle(rle), rlerows(0), grlerows(rlerows), rlelength(0),
     monitorptr(0)
 {
   G_TRY
@@ -207,6 +211,12 @@ GBitmap::GBitmap(const GBitmap &ref, const GRect &rect, int border)
 void 
 GBitmap::init(int arows, int acolumns, int aborder)
 {
+  size_t np = arows * (acolumns + aborder) + aborder;
+  if (arows != (unsigned short) arows ||
+      acolumns != (unsigned short) acolumns ||
+      acolumns + aborder != (unsigned short)(acolumns + aborder) ||
+      (arows > 0 && (np-aborder)/(size_t)arows!=(size_t)(acolumns+aborder)) )
+    G_THROW("GBitmap: image size exceeds maximum (corrupted file?)");
   GMonitorLock lock(monitor());
   destroy();
   grays = 2;
@@ -467,7 +477,7 @@ GBitmap::share()
 {
   if (!monitorptr)
     {
-      unsigned long x = (unsigned long)this;
+      size_t x = (size_t)this;
       monitorptr = &monitors[(x^(x>>5)) % NMONITORS];
     }
 }
@@ -886,11 +896,13 @@ GBitmap::read_rle_raw(ByteStream &bs)
   int c = 0;
   while (n >= 0)
     {
-      bs.read(&h, 1);
+      if (bs.read(&h, 1) <= 0)
+        G_THROW( ByteStream::EndOfFile );
       int x = h;
       if (x >= (int)RUNOVERFLOWVALUE)
         {
-          bs.read(&h, 1);
+          if (bs.read(&h, 1) <= 0)
+            G_THROW( ByteStream::EndOfFile );
           x = h + ((x - (int)RUNOVERFLOWVALUE) << 8);
         }
       if (c+x > ncolumns)
@@ -1278,10 +1290,12 @@ GBitmap::decode(unsigned char *runs)
   // initialize pixel array
   if (nrows==0 || ncolumns==0)
     G_THROW( ERR_MSG("GBitmap.not_init") );
+  if (ncolumns + border != (unsigned short)(ncolumns+border))
+    G_THROW("GBitmap: image size exceeds maximum (corrupted file?)");
   bytes_per_row = ncolumns + border;
   if (runs==0)
     G_THROW( ERR_MSG("GBitmap.null_arg") );
-  int npixels = nrows * bytes_per_row + border;
+  size_t npixels = nrows * bytes_per_row + border;
   if (!bytes_data)
   {
     gbytes_data.resize(npixels);
