@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: GContainer.h,v 1.21 2008/01/07 11:48:52 leonb Exp $
-// $Name: release_3_5_22 $
 
 #ifndef _GCONTAINER_H_
 #define _GCONTAINER_H_
@@ -131,8 +128,7 @@ namespace DJVU {
     @author 
     L\'eon Bottou <leonb@research.att.com> -- initial implementation.\\
     Andrei Erofeev <eaf@geocities.com> -- bug fixes.
-    @version 
-    #$Id: GContainer.h,v 1.21 2008/01/07 11:48:52 leonb Exp $# */
+*/
 //@{
 
 
@@ -182,7 +178,7 @@ hash(const unsigned long & x)
 static inline unsigned int 
 hash(const void * const & x) 
 { 
-  return (unsigned long) x; 
+  return (unsigned int)(size_t) x; 
 }
 
 /** Hashing function (float). */
@@ -266,7 +262,7 @@ protected:
     static void   init(void* dst, int n) {}
     // Trivial copy constructor
     static void   copy(void* dst, const void* src, int n, int ) 
-      { memcpy(dst, src, SZ*n); }
+      { ::memcpy(dst, src, SZ*n); }
     // Trivial destructor
     static void   fini(void* dst, int n) {}
   };
@@ -554,52 +550,61 @@ public:
 template <class TYPE> void
 GArrayTemplate<TYPE>::sort(int lo, int hi)
 {
-  if (hi <= lo)
-    return;
-  if (hi > hibound || lo<lobound)
-    G_THROW( ERR_MSG("GContainer.illegal_subscript") );
   TYPE *data = (TYPE*)(*this);
-  // Test for insertion sort
-  if (hi <= lo + 50)
+  while(true)
     {
-      for (int i=lo+1; i<=hi; i++)
+      if (hi <= lo)
+        return;
+      if (hi > hibound || lo<lobound)
+        G_THROW( ERR_MSG("GContainer.illegal_subscript") );
+      // Test for insertion sort
+      if (hi <= lo + 50)
         {
-          int j = i;
-          TYPE tmp = data[i];
-          while ((--j>=lo) && !(data[j]<=tmp))
-            data[j+1] = data[j];
-          data[j+1] = tmp;
+          for (int i=lo+1; i<=hi; i++)
+            {
+              int j = i;
+              TYPE tmp = data[i];
+              while ((--j>=lo) && !(data[j]<=tmp))
+                data[j+1] = data[j];
+              data[j+1] = tmp;
+            }
+          return;
         }
-      return;
-    }
-  // -- determine suitable quick-sort pivot
-  TYPE tmp = data[lo];
-  TYPE pivot = data[(lo+hi)/2];
-  if (pivot <= tmp)
-    { tmp = pivot; pivot=data[lo]; }
-  if (data[hi] <= tmp)
-    { pivot = tmp; }
-  else if (data[hi] <= pivot)
-    { pivot = data[hi]; }
-  // -- partition set
-  int h = hi;
-  int l = lo;
-  while (l < h)
-    {
-      while (! (pivot <= data[l])) l++;
-      while (! (data[h] <= pivot)) h--;
-      if (l < h)
+      // -- determine median-of-three pivot
+      TYPE tmp = data[lo];
+      TYPE pivot = data[(lo+hi)/2];
+      if (pivot <= tmp)
+        { tmp = pivot; pivot=data[lo]; }
+      if (data[hi] <= tmp)
+        { pivot = tmp; }
+      else if (data[hi] <= pivot)
+        { pivot = data[hi]; }
+      // -- partition set
+      int h = hi;
+      int l = lo;
+      while (l < h)
         {
-          tmp = data[l];
-          data[l] = data[h];
-          data[h] = tmp;
-          l = l+1;
-          h = h-1;
+          while (! (pivot <= data[l])) l++;
+          while (! (data[h] <= pivot)) h--;
+          if (l < h)
+            {
+              tmp = data[l];
+              data[l] = data[h];
+              data[h] = tmp;
+              l = l+1;
+              h = h-1;
+            }
+        }
+      // -- recurse, small partition first
+      //    tail-recursion elimination
+      if (h - lo <= hi - l) {
+        sort(lo,h);
+        lo = l; // sort(l,hi)
+      } else {
+        sort(l,hi);
+        hi = h; // sort(lo,h)
       }
     }
-  // -- recursively restart
-  sort(lo, h);
-  sort(l, hi);
 }
 
 template<class TYPE> inline TYPE&
@@ -833,9 +838,9 @@ public:
 template<class TI>
 class GListImpl : public GListBase
 {
+  typedef GCONT ListNode<TI> LNode;
 protected:
   GListImpl();
-  typedef GCONT ListNode<TI> LNode;
   static Node * newnode(const TI &elt);
   int operator==(const GListImpl<TI> &l2) const;
   int search(const TI &elt, GPosition &pos) const;
@@ -852,7 +857,7 @@ GListImpl<TI>::newnode(const TI &elt)
 {
   LNode  *n = (LNode *) operator new (sizeof(LNode ));
 #if GCONTAINER_ZERO_FILL
-  memset(n, 0, sizeof(LNode ));
+  memset((void*)n, 0, sizeof(LNode ));
 #endif
   new ((void*)&(n->val)) TI(elt);
   return (Node*) n;
@@ -888,6 +893,7 @@ GListImpl<TI>::search(const TI &elt, GPosition &pos) const
 template <class TYPE, class TI>
 class GListTemplate : protected GListImpl<TI>
 {
+  typedef GCONT ListNode<TI> LNode;
 public:
   // -- ACCESS
   /** Returns the number of elements in the list. */
@@ -908,7 +914,7 @@ public:
       segmentation violation. See \Ref{GPosition} for efficient operations on
       positions. */
   TYPE& operator[](GPosition pos)
-    { return (TYPE&) (((typename GListImpl<TI>::LNode *)pos.check((void*)this))->val); }
+    { return (TYPE&) (((LNode *)pos.check((void*)this))->val); }
   /** Returns a constant reference to the list element at position #pos#.
       This reference only be used for reading a list element.  An exception
       \Ref{GException} is thrown if #pos# is not a valid position. This
@@ -916,7 +922,7 @@ public:
       GList<TYPE>#.  See \Ref{GPosition} for efficient operations on
       positions. */
   const TYPE& operator[](GPosition pos) const
-    { return (const TYPE&) (((const typename GListImpl<TI>::LNode *)pos.check((void*)this))->val); }
+    { return (const TYPE&) (((const LNode *)pos.check((void*)this))->val); }
   // -- TEST
   /** Tests whether a list is empty.  
       Returns a non zero value if the list contains no elements. */
@@ -962,21 +968,21 @@ public:
   /** Inserts an element after the last element of the list. 
       The new element is initialized with a copy of argument #elt#. */
   void append(const TYPE &elt)
-    { GListImpl<TI>::append(newnode((const TI&)elt)); }
+    { GListImpl<TI>::append(this->newnode((const TI&)elt)); }
   /** Inserts an element before the first element of the list. 
       The new element is initialized with a copy of argument #elt#. */
   void prepend(const TYPE &elt)
-    { GListImpl<TI>::prepend(newnode((const TI&)elt)); }
+    { GListImpl<TI>::prepend(this->newnode((const TI&)elt)); }
   /** Inserts a new element after the list element at position #pos#.  When
       position #pos# is null the element is inserted at the beginning of the
       list.  The new element is initialized with a copy of #elt#. */
   void insert_after(GPosition pos, const TYPE &elt)
-    { GListImpl<TI>::insert_after(pos, newnode((const TI&)elt)); }
+    { GListImpl<TI>::insert_after(pos, this->newnode((const TI&)elt)); }
   /** Inserts a new element before the list element at position #pos#. When
       position #pos# is null the element is inserted at the end of the
       list. The new element is initialized with a copy of #elt#. */
   void insert_before(GPosition pos, const TYPE &elt)
-    { GListImpl<TI>::insert_before(pos, newnode((const TI&)elt)); }
+    { GListImpl<TI>::insert_before(pos, this->newnode((const TI&)elt)); }
   /** Inserts an element of another list into this list.  This function
       removes the element at position #frompos# in list #frompos#, inserts it
       in the current list before the element at position #pos#, and advances
@@ -1103,10 +1109,10 @@ public:
 template <class K>
 class GSetImpl : public GSetBase
 {
+  typedef GCONT SetNode<K> SNode;
 protected:
   GSetImpl();
   GSetImpl(const Traits &traits);
-  typedef GCONT SetNode<K> SNode;
   HNode *get(const K &key) const;
   HNode *get_or_throw(const K &key) const;
   HNode *get_or_create(const K &key);
@@ -1164,7 +1170,7 @@ GSetImpl<K>::get_or_create(const K &key)
   if (m) return m;
   SNode *n = (SNode*) operator new (sizeof(SNode));
 #if GCONTAINER_ZERO_FILL
-  memset(n, 0, sizeof(SNode));
+  memset((void*)n, 0, sizeof(SNode));
 #endif
   new ((void*)&(n->key)) K ( key );
   n->hashcode = hash((const K&)(n->key));
@@ -1175,10 +1181,10 @@ GSetImpl<K>::get_or_create(const K &key)
 template <class K, class TI>
 class GMapImpl : public GSetImpl<K>
 {
+  typedef GCONT MapNode<K,TI> MNode;
 protected:
   GMapImpl();
   GMapImpl(const GCONT Traits &traits);
-  typedef GCONT MapNode<K,TI> MNode;
   GCONT HNode* get_or_create(const K &key);
 };
 
@@ -1197,16 +1203,16 @@ GMapImpl<K,TI>::GMapImpl(const GCONT Traits &traits)
 template<class K, class TI> GCONT HNode *
 GMapImpl<K,TI>::get_or_create(const K &key)
 {
-  GCONT HNode *m = get(key);
+  GCONT HNode *m = this->get(key);
   if (m) return m;
   MNode *n = (MNode*) operator new (sizeof(MNode));
 #if GCONTAINER_ZERO_FILL
-  memset(n, 0, sizeof(MNode));
+  memset((void*)n, 0, sizeof(MNode));
 #endif
   new ((void*)&(n->key)) K  (key);
   new ((void*)&(n->val)) TI ();
   n->hashcode = hash((const K&)(n->key));
-  installnode(n);
+  this->installnode(n);
   return n;
 }
 
@@ -1221,6 +1227,7 @@ GMapImpl<K,TI>::get_or_create(const K &key)
 template <class KTYPE, class VTYPE, class TI>
 class GMapTemplate : protected GMapImpl<KTYPE,TI>
 {
+  typedef GCONT MapNode<KTYPE,TI> MNode;
 public:
   /** Returns the number of elements in the map. */
   int size() const
@@ -1253,32 +1260,32 @@ public:
       #pos#.  An exception \Ref{GException} is thrown if position #pos# is not
       valid.  There is no direct way to change the key of a map entry. */
   const KTYPE &key(const GPosition &pos) const
-    { return (const KTYPE&)(((typename GMapImpl<KTYPE,TI>::MNode*)(pos.check((void*)this)))->key); }
+    { return (const KTYPE&)(((MNode*)(pos.check((void*)this)))->key); }
   /** Returns a reference to the value of the map entry at position #pos#.
       This reference can be used for both reading (as "#a[n]#") and modifying
       (as "#a[n]=v#").  An exception \Ref{GException} is thrown if position
       #pos# is not valid. */
   VTYPE& operator[](const GPosition &pos)
-    { return (VTYPE&)(((typename GMapImpl<KTYPE,TI>::MNode*)(pos.check((void*)this)))->val); }
+    { return (VTYPE&)(((MNode*)(pos.check((void*)this)))->val); }
   /** Returns a constant reference to the value of the map entry at position
       #pos#.  This reference can only be used for reading (as "#a[n]#") the
       entry value.  An exception \Ref{GException} is thrown if position #pos#
       is not valid. */
   const VTYPE& operator[](const GPosition &pos) const
-    { return (const VTYPE&)(((typename GMapImpl<KTYPE,TI>::MNode*)(pos.check((void*)this)))->val); }
+    { return (const VTYPE&)(((MNode*)(pos.check((void*)this)))->val); }
   /** Returns a constant reference to the value of the map entry for key
       #key#.  This reference can only be used for reading (as "#a[n]#") the
       entry value.  An exception \Ref{GException} is thrown if no entry
       contains key #key#. This variant of #operator[]# is necessary when
       dealing with a #const GMAP<KTYPE,VTYPE>#. */
   const VTYPE& operator[](const KTYPE &key) const
-    { return (const VTYPE&)(((const typename GMapImpl<KTYPE,TI>::MNode*)(get_or_throw(key)))->val); }
+    { return (const VTYPE&)(((const MNode*)(this->get_or_throw(key)))->val); }
   /** Returns a reference to the value of the map entry for key #key#.  This
       reference can be used for both reading (as "#a[n]#") and modifying (as
       "#a[n]=v#"). If there is no entry for key #key#, a new entry is created
       for that key with the null constructor #VTYPE::VTYPE()#. */
   VTYPE& operator[](const KTYPE &key)
-    { return (VTYPE&)(((typename GMapImpl<KTYPE,TI>::MNode*)(get_or_create(key)))->val); }
+    { return (VTYPE&)(((MNode*)(this->get_or_create(key)))->val); }
   /** Destroys the map entry for position #pos#.  
       Nothing is done if position #pos# is not a valid position. */
   void del(GPosition &pos)
