@@ -52,9 +52,6 @@
 //C- | TO ANY WARRANTY OF NON-INFRINGEMENT, OR ANY IMPLIED WARRANTY OF
 //C- | MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 //C- +------------------------------------------------------------------
-// 
-// $Id: c44.cpp,v 1.14 2008/01/11 14:46:56 leonb Exp $
-// $Name: release_3_5_22 $
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -210,8 +207,7 @@
     DjVu IW44 wavelet encoder.
     @author
     L\'eon Bottou <leonb@research.att.com>
-    @version
-    #$Id: c44.cpp,v 1.14 2008/01/11 14:46:56 leonb Exp $# */
+*/
 //@{
 //@}
 
@@ -226,11 +222,7 @@
 #include "GURL.h"
 #include "DjVuMessage.h"
 #include "JPEGDecoder.h"
-
-#include <locale.h>
-#include <stdio.h>
-#include <string.h>
-#include <stdlib.h>
+#include "common.h"
 
 // command line data
 
@@ -335,7 +327,7 @@ parse_bpp(const char *q)
         G_THROW( ERR_MSG("c44.bitrate_comma_expected") );
       q = (*ptr ? ptr+1 : ptr);
       argv_bpp[argc_bpp++] = (float)x;
-      if (argc_bpp>MAXCHUNKS)
+      if (argc_bpp>=MAXCHUNKS)
         G_THROW( ERR_MSG("c44.bitrate_too_many") );
     }
   if (argc_bpp < 1)
@@ -691,8 +683,7 @@ create_photo_djvu_file(IW44Image &iw, int w, int h,
 int
 main(int argc, char **argv)
 {
-  setlocale(LC_ALL,"");
-  djvu_programname(argv[0]);
+  DJVU_LOCALE;
   GArray<GUTF8String> dargv(0,argc-1);
   for(int i=0;i<argc;++i)
     dargv[i]=GNativeString(argv[i]);
@@ -705,7 +696,7 @@ main(int argc, char **argv)
       ByteStream &ibs=*gibs;
       char prefix[16];
       memset(prefix, 0, sizeof(prefix));
-      if (ibs.read((void*)prefix, sizeof(prefix)) < 0)
+      if (ibs.readall((void*)prefix, sizeof(prefix)) < sizeof(prefix))
         G_THROW( ERR_MSG("c44.failed_pnm_header") );
 #ifdef DEFAULT_JPEG_TO_HALF_SIZE
       // Default specification for jpeg files
