@@ -133,11 +133,7 @@
 #include "DjVuDocEditor.h"
 #include "GOS.h"
 #include "DjVuMessage.h"
-
-#include <locale.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <stdlib.h>
+#include "common.h"
 
 static const char * progname;
 
@@ -271,7 +267,11 @@ list(GArray<GUTF8String> &argv)
 	 if (file->is_page())
 	 {
 	    char buffer[128];
+#if HAVE_SNPRINTF
+	    snprintf(buffer, sizeof(buffer), "PAGE #%d", file->get_page_num()+1);
+#else
 	    sprintf(buffer, "PAGE #%d", file->get_page_num()+1);
+#endif
 	    printf("%s", buffer);
 	    for(int i=strlen(buffer);i<9;i++)
 	       putchar(' ');
@@ -296,9 +296,7 @@ list(GArray<GUTF8String> &argv)
 int
 main(int argc, char ** argv)
 {
-  setlocale(LC_ALL,"");
-  setlocale(LC_NUMERIC,"C");
-  djvu_programname(argv[0]);
+  DJVU_LOCALE;
   GArray<GUTF8String> dargv(0,argc-1);
   for(int i=0;i<argc;++i)
     dargv[i]=GNativeString(argv[i]);
